#!/bin/sh

# A slow three wire bus written in shell.
# So far depending on the version of 'cut' from coreutils
# – not working with the busybox applet of 'cut' due to missing features
# Requires 'bc' calculator for hex/binary conversion
#
# Written by Elektra Wagenrad in 2014
#
# ENX = Serial interface select, active low 
# SCLK = Serial clock
# SDATA = Serial data, this port is bidirectional. 
# ENBL = Enable analog function of the chip, needs to be set up/down when changing frequency

ENXPIN=20
SCLKPIN=19
SDATAPIN=22
ENBLPIN=21
MODEPIN=23

# Time in microseconds
TIME=5000

# If we don't receive any parameters, show help

if [ -z "$1" ]
then 
echo 
echo "Configure RF-2501 mixer chip via 3-wire bus"
echo
echo "  Syntax:"
echo
echo "three-wire.sh r <two digit hex address> (r = read)"
echo "three-wire.sh w <two digit hex address> <four digit hex value> (w = write)" 
echo "IMPORTANT: Always write all digits, so 05 instead of 5, 0123 instead of 123"   
echo
exit 0
fi

# Do we have to export the required GPIOs?

if [ ! -e /sys/class/gpio/gpio$SCLKPIN/direction ]
then
echo $ENXPIN > /sys/class/gpio/export
echo $SCLKPIN > /sys/class/gpio/export
echo $SDATAPIN > /sys/class/gpio/export
echo $ENBLPIN > /sys/class/gpio/export
echo $MODEPIN > /sys/class/gpio/export
fi

# Did exporting the SCLKPIN work? If not, bail out

if [ ! -e /sys/class/gpio/gpio$SCLKPIN/direction ]
then
echo 
echo "ERROR: Something is going wrong here. Can't control GPIOs"
echo "Possible reason: GPIOs may not be supported on this platform"
echo
exit 1
fi

# Could check for accessibility of these GPIOs, too.
# Not implemented, yet.
# If this goes out to the world, it might be worth doing. 

echo "out" > /sys/class/gpio/gpio$SCLKPIN/direction
echo "out" > /sys/class/gpio/gpio$ENXPIN/direction
echo "out" > /sys/class/gpio/gpio$ENBLPIN/direction
echo 1 > /sys/class/gpio/gpio$ENBLPIN/value


BINADDRESS=$(echo "obase=2; ibase=16; 1$2" | bc | cut -c 2- )

echo "BINADDRESS:  $BINADDRESS"

if [ "$1" = w ]

then

echo
echo "Write mode"

# Initialize write process:
                                                
BINDATA=$(echo "obase=2; ibase=16; 1$3" | bc | cut -c 2- )

echo "out" > /sys/class/gpio/gpio$SDATAPIN/direction
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo 1 > /sys/class/gpio/gpio$ENXPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo 0 > /sys/class/gpio/gpio$ENXPIN/value
echo 1 > /sys/class/gpio/gpio$SDATAPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SDATAPIN/value
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME

# Send 7 address bits

for highlow in `echo $BINADDRESS | cut -c 2,3,4,5,6,7,8  --output-delimiter=\  `

do
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo $highlow >  /sys/class/gpio/gpio$SDATAPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
done

# Send 16 data bits

for highlow in `echo $BINDATA | cut -c 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16  --output-delimiter=\  `

do
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo $highlow >  /sys/class/gpio/gpio$SDATAPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
done

echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo 1 > /sys/class/gpio/gpio$ENXPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value

# End of write process

fi


if [ "$1" = r ]

then
echo
echo "Read mode"

# Initialize read process:

echo "in" > /sys/class/gpio/gpio$SDATAPIN/direction
echo 1 > /sys/class/gpio/gpio$ENXPIN/value
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
usleep $TIME
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value &
echo 0 > /sys/class/gpio/gpio$ENXPIN/value


usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value


usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo "out" > /sys/class/gpio/gpio$SDATAPIN/direction                            
echo 1 > /sys/class/gpio/gpio$SDATAPIN/value

usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME

# Send 7 address bits

for highlow in `echo $BINADDRESS | cut -c 2,3,4,5,6,7,8  --output-delimiter=\  `

do
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
echo $highlow >  /sys/class/gpio/gpio$SDATAPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
done



# Read 16 data bits
echo "in" > /sys/class/gpio/gpio$SDATAPIN/direction
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME


rm /tmp/rfmd.txt 2> /dev/null
COUNTER=1

MAX=17

while [ "$COUNTER" != "$MAX" ];

do 

echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
cat /sys/class/gpio/gpio$SDATAPIN/value >> /tmp/rfmd.txt
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
let "COUNTER += 1"

done

echo 1 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 0 > /sys/class/gpio/gpio$SCLKPIN/value
usleep $TIME
echo 1 > /sys/class/gpio/gpio$ENXPIN/value


RESULT=`cat /tmp/rfmd.txt  | sed 'N;N;N;N;N;N;N;N;N;N;N;N;N;N;N ;s/\n//g'`
echo
echo "Content of address "$2 "(hex) :"
echo
echo "     `echo  "obase=16; ibase=2; "$RESULT"" | bc` "
echo
echo "Bits:"
echo
echo " 16  15  14  13  12  11  10  09  08  07  06  05  04  03  02  01"
echo
echo "  `cat /tmp/rfmd.txt  | sed 'N;N;N;N;N;N;N;N;N;N;N;N;N;N;N ;s/\n/   /g'`"
echo

fi


